/*###############################################################################
## ETiles project
##
## Copyright (C) 2010 Carlos Villegas Núñez  carlos.villegasnunez@alum.uca.es
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
###############################################################################*/

#ifndef PROPMAPA_H
#define PROPMAPA_H

#include <QDialog>
#include <QColorDialog>
#include <QPalette>

#include "ventanaprincipal.h"

namespace Ui {
    class propmapa;
}

class propmapa : public QDialog {
    Q_OBJECT
public:
    /** Constructor de la clase propmapa, que se encargar de inicializar los componentes que formaran el diálogo propiedades
    * @param parent: Puntero a la instancia que se encargar de realizar la creación
    * @param crejilla: Color de la rejilla
    * @param cmapa: Color de fondo del mapa
    * @param csel: Color del cuadro de seleccion
    */
    propmapa(QWidget *parent = 0, QColor crejilla=0, QColor cmapa=0, QColor csel=0);
    /** Destructor de la clase propmapa*/
    ~propmapa();

protected:
    /** Procedimiento que se encarga de cambiar de evento
     * @param e: Parámetro que identifica el evento producido
     */
    void changeEvent(QEvent *e);

private:
    Ui::propmapa *ui;
    QColorDialog diar,diaf,dias;
    QColor cr,cf,cs;

private slots:
    /** Procedimiento ligado a la acción sobre el botón cancelar, indicando que se cancela el cambio de propiedades*/
    void on_buttonBox_rejected();
    /** Procedimiento ligado a la acción sobre el botón aceptar, indicando que se apliquen los cambios en las propiedades*/
    void on_buttonBox_accepted();
    /** Procedimiento ligado a la acción sobre el botón del color de fondo, indicando que se abra el diálogo de color*/
    void on_elegirf_clicked();
    /** Procedimiento ligado a la acción sobre el botón del color del cuadro de seleccion, indicando que se abra el diálogo de color*/
    void on_elegirs_clicked();
    /** Procedimiento ligado a la acción sobre el botón del color de la rejilla, indicando que se abra el diálogo de color*/
    void on_elegirr_clicked();
};

#endif // PROPMAPA_H
