/*###############################################################################
## ETiles project
##
## Copyright (C) 2010 Carlos Villegas Núñez  carlos.villegasnunez@alum.uca.es
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
###############################################################################*/

#include <QtGui>
#include <iostream>
#include <fstream>
#include <stdlib.h>
#include <QBuffer>
#include <QVBoxLayout>

#include "ventanaprincipal.h"
#include "ui_ventanaprincipal.h"
#include "dnuevoproyecto.h"
#include "cargartileset.h"
#include "redimensionar.h"
#include "propmapa.h"
#include "perlin.h"

using namespace std;

VentanaPrincipal::VentanaPrincipal(QWidget *parent)
    : QMainWindow(parent), ui(new Ui::VentanaPrincipal)
{
    ui->setupUi(this);

    inicio=0;
    inicioguardado=0;

    //Caracteristicas de la ventana principal
    setFixedSize(800,600);
    setMaximumSize(4000,4000);
    setMinimumSize(200,200);
    move(100,100);

    //Acciones de la ventana principal
    ui->mainToolBar->setWindowTitle("Barra de herramientas");
    nuevo = new QAction(QIcon("../imagenes/iconos/32x32/nuevo32.png"),tr("Nuevo"),this);
    connect(nuevo,SIGNAL(triggered()),this,SLOT(on_actionNormal_activated()));
    ui->mainToolBar->addAction(nuevo);

    guardar = new QAction(QIcon("../imagenes/iconos/32x32/guardar32.png"),tr("Guardar"),this);
    connect(guardar,SIGNAL(triggered()),this,SLOT(on_actionGuardar_activated()));
    ui->mainToolBar->addAction(guardar);

    cargar = new QAction(QIcon("../imagenes/iconos/32x32/abrir32.png"),tr("Cargar"),this);
    connect(cargar,SIGNAL(triggered()),this,SLOT(on_actionCargar_activated()));
    ui->mainToolBar->addAction(cargar);

    ui->mainToolBar->addSeparator();

    deshacer = new QAction(QIcon("../imagenes/iconos/32x32/deshacer32.png"),tr("Deshacer"),this);
    connect(deshacer,SIGNAL(triggered()),this,SLOT(on_actionDeshacer_activated()));
    ui->mainToolBar->addAction(deshacer);

    rehacer = new QAction(QIcon("../imagenes/iconos/32x32/rehacer32.png"),tr("Rehacer"),this);
    connect(rehacer,SIGNAL(triggered()),this,SLOT(on_actionRehacer_activated()));
    ui->mainToolBar->addAction(rehacer);

    ui->mainToolBar->addSeparator();

    zoommenos = new QAction(QIcon("../imagenes/iconos/32x32/lupamenos32.png"),tr("Disminuir Zoom"),this);
    connect(zoommenos,SIGNAL(triggered()),this,SLOT(on_actionZoomOut_activated()));
    ui->mainToolBar->addAction(zoommenos);

    zoommas = new QAction(QIcon("../imagenes/iconos/32x32/lupamas32.png"),tr("Aumentar Zoom"),this);
    connect(zoommas,SIGNAL(triggered()),this,SLOT(on_actionZoomIn_activated()));
    ui->mainToolBar->addAction(zoommas);

    ui->mainToolBar->addSeparator();

    seleccion = new QAction(QIcon("../imagenes/iconos/32x32/seleccion32.png"),tr("Selección"),this);
    connect(seleccion,SIGNAL(triggered()),this,SLOT(SeleccionActivada()));
    ui->mainToolBar->addAction(seleccion);

    borrar = new QAction(QIcon("../imagenes/iconos/32x32/borrar32.png"),tr("Borrar"),this);
    connect(borrar,SIGNAL(triggered()),this,SLOT(BorrarActivado()));
    ui->mainToolBar->addAction(borrar);

    relleno = new QAction(QIcon("../imagenes/iconos/32x32/pintar32.png"),tr("Relleno"),this);
    connect(relleno,SIGNAL(triggered()),this,SLOT(RellenoActivado()));
    ui->mainToolBar->addAction(relleno);

    rellenoConjunto = new QAction(QIcon("../imagenes/iconos/32x32/rellenar32.png"),tr("Relleno Conjunto"),this);
    connect(rellenoConjunto,SIGNAL(triggered()),this,SLOT(RellenoConjuntoActivado()));
    ui->mainToolBar->addAction(rellenoConjunto);

    /*Inicialización de las partes del editor*/

    //Interfaz Mapa
    scrollAreamapa = new MapaScrollArea;
    setCentralWidget(scrollAreamapa);

    //Interfaz Minimapa
    areaminimapa = new QDockWidget("Minimapa");
    addDockWidget(Qt::RightDockWidgetArea,areaminimapa);
    ui->menuVista->addAction(areaminimapa->toggleViewAction());

    //Interfaz Lista Tiles
    listatiles = new ListaTiles();

    QVBoxLayout *cvertical;

    cvertical = new QVBoxLayout;
    cvertical->addWidget(listatiles);
    cvertical->addWidget(listatiles->BarraAcciones());

    QWidget *contenedorareatiles = new QWidget();
    contenedorareatiles->setLayout(cvertical);

    areatiles = new QDockWidget("Tiles");
    areatiles->setWidget(contenedorareatiles);

    addDockWidget(Qt::RightDockWidgetArea,areatiles);
    ui->menuVista->addAction(areatiles->toggleViewAction());

    connect(listatiles->AccionEliminar(),SIGNAL(triggered()),this,SLOT(EliminarTileArea()));
    connect(listatiles->AccionAnadir(),SIGNAL(triggered()),this,SLOT(on_actionImportar_activated()));
    connect(listatiles->AccionTileset(),SIGNAL(triggered()),this,SLOT(AnadirTileset()));
}

VentanaPrincipal::~VentanaPrincipal()
{
    delete listatiles;
    if(inicio)
    {
        delete tile;
        delete mapa;
        delete minimapa;
    }
    delete ui;
    delete nuevo; delete guardar; delete cargar; delete deshacer; delete rehacer; delete seleccion;
    delete borrar; delete relleno; delete zoommas; delete zoommenos;
}

void VentanaPrincipal::setMensageEstado(QString msg)
{
     ui->statusBar->showMessage(msg);
}

void VentanaPrincipal::on_actionNormal_activated()
{
    if(inicio==1)
    {
        if(mapa->ModificadoDesdeGuardado() || tile->ModificadoDesdeGuardado())
        {
            QMessageBox msgBox(QMessageBox::Warning, tr("¡Cuidado!"),
                               "El mapa actual ha sido modificado, ¿Quiere guardar los cambios realizados?", 0, this);
            msgBox.addButton(tr("Guardar"), QMessageBox::AcceptRole);
            msgBox.addButton(tr("Cancelar"), QMessageBox::RejectRole);
            msgBox.addButton(tr("No Guardar"), QMessageBox::NoRole);
            int resp = msgBox.exec();
            if (resp == QMessageBox::AcceptRole)
            {
                GuardarProyecto(0);
            }
            else if(resp == QMessageBox::RejectRole)
            {
                return;
            }
        }      
    }
    DNuevoProyecto dnproyecto(this);
    if(dnproyecto.exec() == QDialog::Accepted)
    {
        CrearVentanaTiles();
        CrearMinimapa();
        inicio=1;
    }
}

void VentanaPrincipal::on_actionAleatorio_activated()
{
    maleatorios dma(this);

    dma.exec();
}

void VentanaPrincipal::on_actionCargar_activated()
{
    CargarProyecto();
}

void VentanaPrincipal::on_actionGuardar_activated()
{
    if(inicio == 1)
    {
        if(tile->ModificadoDesdeGuardado() || mapa->ModificadoDesdeGuardado())
        {
            GuardarProyecto(0);
        }
    }
}

void VentanaPrincipal::on_actionGuardar_Como_activated()
{
    GuardarProyecto(1);
}

void VentanaPrincipal::on_actionExportar_activated()
{
    if(inicio==1)
    {
        Exportar();
    }
}

void VentanaPrincipal::on_actionSalir_activated()
{
    if(inicio == 1)
    {
        if(mapa->ModificadoDesdeGuardado() || tile->ModificadoDesdeGuardado())
        {
            QMessageBox msgBox(QMessageBox::Warning, tr("¡Atención!"),
                               "El mapa actual ha sido modificado, ¿Quiere guardar los cambios realizados?", 0, this);
            msgBox.addButton(tr("Guardar"), QMessageBox::AcceptRole);
            msgBox.addButton(tr("Cancelar"), QMessageBox::RejectRole);
            msgBox.addButton(tr("No Guardar"), QMessageBox::NoRole);
            int resp = msgBox.exec();
            if (resp == QMessageBox::AcceptRole)
            {
                    GuardarProyecto(0);
            }
            else if(resp == QMessageBox::RejectRole)
            {
                return;
            }
        }
    }
    exit(0);
}

void VentanaPrincipal::on_actionCopiar_activated()
{
    if(inicio!=0)
        mapa->Copiar();
}

void VentanaPrincipal::on_actionCortar_activated()
{
    if(inicio!=0)
        mapa->Cortar();
}

void VentanaPrincipal::on_actionPegar_activated()
{
    if(inicio!=0)
        mapa->PegarActivar(true);
}

void VentanaPrincipal::on_actionEliminar_activated()
{
    if(inicio!=0)
        mapa->Eliminar();
}

void VentanaPrincipal::EliminarTileArea()
{
    int sel=listatiles->TileSeleccionado();
    bool upd = false;

    if (sel!= -1)
    {
        if(tile->Cantidad(sel) > 1) //Es un tileSet
        {
            QMessageBox msgBox(QMessageBox::Warning, tr("¡Atención!"),
                               "El tile seleccionado pertenece a un TileSet, ¿está seguro de querer eliminarlo?", 0, this);
            msgBox.addButton(tr("Si"), QMessageBox::AcceptRole);
            msgBox.addButton(tr("No"), QMessageBox::RejectRole);
            int resp = msgBox.exec();
            if (resp == QMessageBox::AcceptRole)
            {
                int primero = tile->PrimeraPosTileSet(sel);
                int ultima = tile->UltimaPosTileSet(sel);
                for(int i=0;i<mapa->NumAnchuraTile();i++)
                    for(int j=0;j<mapa->NumAlturaTile();j++)
                    {
                        for(int k=primero; k<=ultima; k++)
                        {
                            if(mapa->TilePosicion(i,j) == k)
                            {
                                mapa->setTilePosicion(-1,i,j);
                                upd = true;
                            }
                        }
                        if(mapa->TilePosicion(i,j) > ultima)
                        {
                            mapa->setTilePosicion(mapa->TilePosicion(i,j)-(ultima-primero+1),i,j);
                        }
                    }
                for(int k=primero; k<=ultima; k++)
                {
                    mapa->ActualizarDeshacer(k);
                    mapa->ActualizarRehacer(k);
                }
                tile->removeRow(sel);
            }
            else if(resp == QMessageBox::RejectRole)
            {
                return;
            }
        }
        else
        {
            QMessageBox msgBox(QMessageBox::Warning, tr("¡Atención!"),
                               "¿está seguro de querer eliminarlo?", 0, this);
            msgBox.addButton(tr("Si"), QMessageBox::AcceptRole);
            msgBox.addButton(tr("No"), QMessageBox::RejectRole);
            int resp = msgBox.exec();
            if (resp == QMessageBox::AcceptRole)
            {
                for(int i=0;i<mapa->NumAnchuraTile();i++)
                    for(int j=0;j<mapa->NumAlturaTile();j++)
                    {
                        if(mapa->TilePosicion(i,j) == sel)
                        {
                            mapa->setTilePosicion(-1,i,j);
                            upd = true;
                        }
                        else if(mapa->TilePosicion(i,j) > sel)
                        {
                            mapa->setTilePosicion(mapa->TilePosicion(i,j)-1,i,j);
                        }
                    }
                tile->removeRow(sel);
                mapa->ActualizarDeshacer(sel);
            }
            else if(resp == QMessageBox::RejectRole)
            {
                return;
            }
        }
        if(upd)
        {
            minimapa->Pintar();
            mapa->update();
        }
    }
}

void VentanaPrincipal::AnadirTileset()
{
    if(inicio != 0)
    {
        cargartileset ventanac(this,tile,mapa->AnchuraTile(),mapa->AlturaTile());
        ventanac.exec();
    }
}

void VentanaPrincipal::RellenoActivado()
{
    if(inicio!=0){
        if(mapa->RellenoActivo())
        {
            mapa->RellenoActivar(false);
            relleno->setChecked(false);
        }
        else
        {
            if(mapa->BorrarActivado())
            {
                mapa->BorrarActivar(false);
                borrar->setChecked(false);
            }
            else if(mapa->SeleccionActiva())
            {
                mapa->SeleccionActivar(false);
                seleccion->setChecked(false);
            }
            else if(mapa->RellenoConjuntoActivado())
            {
                mapa->RellenoConjuntoActivar(false);
                rellenoConjunto->setChecked(false);
            }
            mapa->RellenoActivar(true);
            relleno->setChecked(true);
        }
    }
}

void VentanaPrincipal::RellenoConjuntoActivado()
{
    if(inicio!=0){
        if(mapa->RellenoConjuntoActivado())
        {
            mapa->RellenoConjuntoActivar(false);
            rellenoConjunto->setChecked(false);
        }
        else
        {
            if(mapa->BorrarActivado())
            {
                mapa->BorrarActivar(false);
                borrar->setChecked(false);
            }
            else if(mapa->SeleccionActiva())
            {
                mapa->SeleccionActivar(false);
                seleccion->setChecked(false);
            }
            else if(mapa->RellenoActivo())
            {
                mapa->RellenoActivar(false);
                relleno->setChecked(false);
            }
            mapa->RellenoConjuntoActivar(true);
            rellenoConjunto->setChecked(true);
        }
    }
}

void VentanaPrincipal::BorrarActivado()
{
    if(inicio!=0){
        if(mapa->BorrarActivado())
        {
            mapa->BorrarActivar(false);
            borrar->setChecked(false);
        }
        else
        {
            if(mapa->RellenoActivo())
            {
                mapa->RellenoActivar(false);
                relleno->setChecked(false);
            }
            else if(mapa->SeleccionActiva())
            {
                mapa->SeleccionActivar(false);
                seleccion->setChecked(false);
            }
            else if(mapa->RellenoConjuntoActivado())
            {
                mapa->RellenoConjuntoActivar(false);
                rellenoConjunto->setChecked(false);
            }
            mapa->BorrarActivar(true);
            borrar->setChecked(true);
        }
    }
}

void VentanaPrincipal::SeleccionActivada()
{
    if(inicio!=0){
        if(mapa->SeleccionActiva())
        {
            mapa->SeleccionActivar(false);
            seleccion->setChecked(false);
        }
        else
        {
            if(mapa->RellenoActivo())
            {
                mapa->RellenoActivar(false);
                relleno->setChecked(false);
            }
            else if(mapa->BorrarActivado())
            {
                mapa->BorrarActivar(false);
                borrar->setChecked(false);
            }
            else if(mapa->RellenoConjuntoActivado())
            {
                mapa->RellenoConjuntoActivar(false);
                rellenoConjunto->setChecked(false);
            }
            mapa->SeleccionActivar(true);
            seleccion->setChecked(true);
        }
    }
}

void VentanaPrincipal::on_actionImportar_activated()
{
    if(inicio != 0)
    {
        QPixmap newImage;
        QString fileName = QFileDialog::getOpenFileName(this);
        if(!fileName.isEmpty())//Se ha cancelado el diálogo
        {
            if (!newImage.load(fileName))
            {
                    QMessageBox::warning(this, tr("Importar Tile"),tr("La imagen no puede ser cargada"),QMessageBox::Cancel);
            }
            else
            {
                tile->addPiece(newImage, fileName);
            }
        }
    }
}

void VentanaPrincipal::on_actionRehacer_activated()
{
    if(inicio!=0)
        mapa->Rehacer();
}

void VentanaPrincipal::on_actionDeshacer_activated()
{
    if(inicio!=0)
        mapa->Deshacer();
}

void VentanaPrincipal::on_actionMapa_activated()
{
    if(inicio!=0)
    {
        if(!ui->actionMapa->isChecked())
            mapa->close();
        else
            mapa->show();
    }
}

void VentanaPrincipal::on_actionRejilla_activated()
{
    if(inicio!=0)
    {
        if(ui->actionRejilla->isChecked())
        {
            mapa->ActivarRejilla(true);
        }
        else
        {
            mapa->ActivarRejilla(false);
        }
    }
}

void VentanaPrincipal::on_actionZoomIn_activated()
{
    if(inicio!=0)
        mapa->Zoom(1);
}

void VentanaPrincipal::on_actionZoomOut_activated()
{
    if(inicio!=0)
        mapa->Zoom(0);
}

void VentanaPrincipal::on_actionRedimensionar_activated()
{
    if(inicio!=0)
    {
        redimensionar red(this);
        red.exec();
    }
}

void VentanaPrincipal::on_actionPropiedades_activated()
{
    if(inicio!=0)
    {
        propmapa prop(this,mapa->ColorRejilla(),mapa->ColorFondo(),mapa->ColorSel());
        prop.exec();
    }
}

void VentanaPrincipal::CrearNuevoMapa(double tanchura,double taltura, double tnumanchura, double tnumaltura)
{
    if(inicio == 1)
        Limpiar();

    relleno->setCheckable(true);
    borrar->setCheckable(true);
    seleccion->setCheckable(true);
    rellenoConjunto->setCheckable(true);

    nombremapa = new QString();
    mapa = new Mapa(this,tanchura,taltura,tnumanchura,tnumaltura);

    //Interfaz Mapa
    scrollAreamapa = new MapaScrollArea;
    setCentralWidget(scrollAreamapa);
    scrollAreamapa->setWidget(mapa);
}

void VentanaPrincipal::AleatorioPerlin(int octaves, double frecuencia, double amplitud, int semilla,double margen)
{
    Perlin *p;
    p = new Perlin(octaves,frecuencia,amplitud,semilla);
    int ntile,tsel;
    double valor=0;
    for(int i=0;i<mapa->NumAnchuraTile();i++)
    {
        for(int j=0;j<mapa->NumAlturaTile();j++)
        {
               valor = p->Get(double(i)/double(mapa->NumAnchuraTile()),double(j)/double(mapa->NumAlturaTile()));
               ntile = 0;
               bool enc=false;
               for(double k=-1+margen; k<=1 && !enc; k += margen)
               {
                   if(valor < k)
                   {
                        tsel=ntile;
                        enc=true;
                   }
                   ntile++;
               }
               mapa->setTilePosicion(tsel,i,j);
        }
    }
}

void VentanaPrincipal::CrearMapaAleatorio(double tanchura, double taltura, double tnumanchura, double tnumaltura, QList<QPixmap>ltiles, QList<QString>lnombres, int aguatierra, int contislas, int monvar, int semilla,int eleccion)
{
    CrearNuevoMapa(tanchura,taltura,tnumanchura,tnumaltura);
    CrearVentanaTiles();
    CrearMinimapa();

    //Si al menos tenemos un tile en la lista.
    if(ltiles.size() > 0)
    {
        //Insertamos los tiles en la lista tiles.
        for(int i=0;i<ltiles.size();i++)
        {
            tile->addPiece(ltiles.at(i),lnombres.at(i));
        }
        //Si la eleccion es el algoritmo Perlin Noise.
        if(eleccion==0)
        {
            double margen=2./ltiles.size();
            int octaves=(int)(aguatierra/20)+4;
            double frecuencia=(contislas/13.)+1;
            double amplitud=(monvar*2-100)/100.;
            AleatorioPerlin(octaves,frecuencia,amplitud,semilla,margen);
        }
        else if(eleccion==1)
        {

        }
    }

    inicio = 1;
    minimapa->Pintar(QRect(0,0,200,200));
}

void VentanaPrincipal::RedimensionarMapa(int tanchura, int taltura, int tnumanchura, int tnumaltura)
{
    mapa->Redimensionar(tanchura,taltura,tnumanchura,tnumaltura);
}

void VentanaPrincipal::CrearVentanaTiles()
{
    tile = new Tile(this,listatiles);

    listatiles->setModel(tile);
    mapa->AsignarListaTile(tile);
}

void VentanaPrincipal::CrearMinimapa()
{
    QSize tam;
    tam.setWidth(centralWidget()->width());
    tam.setHeight(centralWidget()->height());

    minimapa = new Minimapa(mapa,scrollAreamapa);
    scrollminimapa.setWidget(minimapa);
    scrollminimapa.setMinimumSize(205,205);
    areaminimapa->setWidget(&scrollminimapa);

    mapa->setMinimapa(minimapa);
}

void VentanaPrincipal::CargarProyecto()
{
    if(inicio==1)
    {
        if(mapa->ModificadoDesdeGuardado() || tile->ModificadoDesdeGuardado())
        {
            QMessageBox msgBox(QMessageBox::Warning, tr("¡Atención!"),
                               "El mapa actual ha sido modificado, ¿Quiere guardar los cambios realizados?", 0, this);
            msgBox.addButton(tr("Guardar"), QMessageBox::AcceptRole);
            msgBox.addButton(tr("Cancelar"), QMessageBox::RejectRole);
            msgBox.addButton(tr("No Guardar"), QMessageBox::NoRole);
            int resp = msgBox.exec();
            if (resp == QMessageBox::AcceptRole)
            {
                GuardarProyecto(0);
            }
            else if(resp == QMessageBox::RejectRole)
            {
                return;
            }
        }
    }

    QString fileName = QFileDialog::getOpenFileName(this,tr("Cargar Mapa"),"/",tr("Mapa (*.map)"));

    if(!fileName.isEmpty())
    {
        QFile archivo(fileName);

        if(archivo.open(QFile::ReadOnly))
        {
            int tanchura,taltura,tnumanchura,tnumaltura;

            QDataStream dataStream(&archivo);
            dataStream >> tanchura >> tnumanchura >> taltura >> tnumaltura;

            CrearNuevoMapa(tanchura,taltura,tnumanchura,tnumaltura);
            CrearVentanaTiles();
            CrearMinimapa();

            int numtileslista;
            int numtile;
            QPixmap pixmap;
            QString nomb;
            QPoint pos;
            dataStream >> numtileslista;

            for(int i=0; i<numtileslista; i++)
            {
                dataStream >> pixmap;
                dataStream >> nomb;
                tile->addPiece(pixmap,nomb);
            }

            int numtilesmapa;
            dataStream >> numtilesmapa;
            for(int i=0; i<numtilesmapa; i++)
            {
                dataStream >> numtile;
                dataStream >> pos;
                mapa->setTilePosicion(numtile, pos.x(), pos.y());
            }
            mapa->update();
            minimapa->update(QRect(0,0,200,200));
            archivo.close();

            nombremapa = new QString(fileName);
        }
        if(inicio == 0)
        {
            inicio = 1;
            inicioguardado = 1;
        }
    }
}

void VentanaPrincipal::GuardarProyecto(int i)
{
    if(inicio == 1)
    {
        QString fileName;
        if(inicioguardado==0 || i==1)
        {
            fileName = QFileDialog::getSaveFileName(this,tr("Guardar Mapa"),"/",tr("Mapa (*.map)"));
            if(!fileName.isNull())
            {
                nombremapa = new QString(fileName);
                inicioguardado=1;
            }
        }
        else
        {
            fileName = *nombremapa;
        }

        if(!fileName.isEmpty())
        {
            QString formato = fileName.section('.',-1);

            if(formato == "MAP" || formato == "map")
            {
                QFile *archivo;
                archivo = new QFile(fileName);

                if(archivo->open(QFile::WriteOnly))
                {
                    QList<int> ltiles;
                    QList<QPoint> lpos;
                    QDataStream dataStream(archivo);

                    dataStream << mapa->AnchuraTile() << mapa->NumAnchuraTile() << mapa->AlturaTile() << mapa->NumAlturaTile();

                    for(int i=0; i < mapa->NumAnchuraTile(); i++)
                        for(int j=0; j < mapa->NumAlturaTile(); j++)
                        {
                            if(!mapa->posLibre(i,j))
                            {
                                ltiles.push_back(mapa->TilePosicion(i,j));
                                lpos.push_back(QPoint(i,j));
                            }
                        }
                    /*Insertamos el tamaños de la Lista de Tiles y cada uno de ellos.*/
                    dataStream << tile->Tamano();
                    for(int i=0; i < tile->Tamano(); i++)
                    {
                        dataStream << QString(tile->NombreTile(i));
                        dataStream << tile->ImagenTile(i);
                    }

                    dataStream << ltiles.size();
                    for(int i=0; i < ltiles.size(); i++)
                    {
                        dataStream << ltiles.at(i);
                        dataStream << lpos.at(i);
                    }
                    archivo->close();
                }
                mapa->MapaGuardado();
                tile->TileGuardado();
            }
            else
            {
                QMessageBox::warning(this, tr("Guardar Mapa"),tr("Formato incorrecto"),QMessageBox::Cancel);
            }
        }
    }
}

void VentanaPrincipal::Exportar()
{
     if(inicio == 1)
     {
            QString fileName;
            QString fileFormats = "(";
            //QString fileFormats;
                /* Get all inputformats */
                for (int i = 0; i < QImageReader::supportedImageFormats().count(); i++) {
                    //fileFormats += QString(QImageReader::supportedImageFormats().at(i).)
                        fileFormats += "*."; /* Insert wildcard */
                        fileFormats += QString(QImageReader::supportedImageFormats().at(i)).toLower(); /* Insert the format */
                        fileFormats += " "; /* Insert a space */
                }
                //fileFormats += ")";

                //QString fileName = QFileDialog::getOpenFileName(this, tr("Open Image"),currentPath, tr("Images ") + fileFormats);
                fileName = QFileDialog::getSaveFileName(this,tr("Exportar"),"/",tr("Imagenes ") + fileFormats + ";;" + tr("Formato para bibliotecas (*.mbp)"));
                if(!fileName.isEmpty())
                {
                    QString formato = fileName.section('.',-1);
                    if(formato == "mbp")
                    {
                        const char *c = fileName.toLocal8Bit().constData();
                        ofstream fs(c);

                        fs << mapa->AnchuraTile() << endl;
                        fs << mapa->AlturaTile() << endl;
                        fs << mapa->NumAnchuraTile() << endl;
                        fs << mapa->NumAlturaTile() << endl;

                        for(int j=0; j < mapa->NumAlturaTile(); j++)
                        {
                            for(int i=0; i < mapa->NumAnchuraTile(); i++)
                            {

                                   fs << mapa->TilePosicion(i,j);
                                   fs << ',';
                            }
                            fs << endl;
                        }
                        for(int i=0;i < tile->Tamano();i++)
                        {
                            fs << tile->SeparacionTile(i) << ',' << tile->NombreTile(i).toLocal8Bit().constData() << endl;
                        }

                        fs.close();
                    }
                    else
                    {
                        QPixmap *imagenmapa= new QPixmap(mapa->AnchuraTile()*mapa->NumAnchuraTile(),mapa->AlturaTile()*mapa->NumAlturaTile());

                        QPainter painter;
                        painter.begin(imagenmapa);
                        painter.fillRect(0,0,mapa->AnchuraTile()*mapa->NumAnchuraTile(),mapa->AlturaTile()*mapa->NumAlturaTile(),Qt::black);
                        for(int i=0;i<mapa->NumAnchuraTile();i++)
                            for(int j=0; j<mapa->NumAlturaTile();j++)
                            {
                                if(!mapa->posLibre(i,j))
                                    painter.drawPixmap(mapa->RectanguloPosicion(i,j),tile->ImagenTile(mapa->TilePosicion(i,j)));
                            }
                        painter.end();
                        if(!imagenmapa->save(fileName))
                             QMessageBox::warning(this, tr("Exportar"),tr("Formato incorrecto"),QMessageBox::Cancel);
                    }
                 }
      }
}

void VentanaPrincipal::Limpiar()
{
    delete tile;
    delete mapa;
    if(!nombremapa->isEmpty())
        delete nombremapa;
}
